namespace TestWeb.Users;

public static class UserApi
{
    public static void MapEndpoints(IEndpointRouteBuilder endpoints)
    {
        var group = endpoints.MapGroup("/api/users").WithTags("Users").AllowAnonymous();
        group.MapPost("/login", Login).WithSummary("Log in a user").WithRequestValidation<LoginRequest>();
    }

    public record LoginRequest(string Username, string Password);

    public record LoginResponse(string Token);

    public class LoginRequestValidator : AbstractValidator<LoginRequest>
    {
        public LoginRequestValidator()
        {
            RuleFor(x => x.Username).NotEmpty();
            RuleFor(x => x.Password).NotEmpty();
        }
    }

    private static async Task<Results<Ok<LoginResponse>, UnauthorizedHttpResult>> Login(
        LoginRequest request,
        AppDbContext db,
        Jwt jwt,
        CancellationToken ct)
    {
        var user = await db.Users.SingleOrDefaultAsync(x => x.Username == request.Username && x.Password == request.Password, ct);

        if (user is null || user.Password != request.Password)
        {
            return TypedResults.Unauthorized();
        }

        var token = jwt.GenerateToken(user);
        var response = new LoginResponse(token);
        return TypedResults.Ok(response);
    }
}
